package derez.libs;

import java.text.ParseException;

import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.keywords.DateTime;


/**
 * Converts Gregorian\Julian dates to Hebrew dates and vice versa.
 */

@ShortName("DateConvert")
@Author("Derez")
@Version((float)1.22)

public class DateConvert {
	
	
	/**
	 * Returns the Hebrew date.
	 * If HebText is true, the returned string will be in Hebrew, else - numerical.
	 * The numbers of Adar A and Adar B are 6 and 7 in a leap year.
	 * In regular year they are both named Adar.
	 * The result include the thousands.
	 * Year must be > 1 . 
	 */
	public String GregToHeb( int day, int month, int year, boolean HebText) throws ParseException 
	{   
		if ((day < 1 )|(day > 31)| (month<1)| (month>12)| (year <2))
			{return "-1";}
		else if ((year == 1582) & (month == 10) & (day > 4) & (day < 15))
			{return "-1";}
		else
		{
		DateTime.setDateFormat("dd/MM/yyyy") ;
		long dGreg = DateTime.DateParse("01/01/2000") ;
		dGreg = DateTime.Add(dGreg, year-2000, month-1, day -1 + deltaG(day, month, year)) 
				+ DateTime.TicksPerHour ; 
		return GToH(dGreg, HebText);
		}
	}
	
	/**
	 * Returns the Hebrew date.
	 * If HebText is true, the returned string will be in Hebrew, else - numerical.
	 * The numbers of Adar A and Adar B are 6 and 7 in a leap year.
	 * In regular year they are both named Adar.
	 * The result includes the thousands.
	 * The parameter GragDate is the Gregorian date in ticks. 
	 */
	public String GregToHeb2( long GregDate , boolean HebText) throws ParseException 
	{   
		//int day,month,year ;
		GregDate += DateTime.TicksPerHour ;
	/*	day = DateTime.GetDayOfMonth(GregDate);
		month = DateTime.GetMonth(GregDate);
		year = DateTime.GetYear(GregDate);*/
		return GToH(GregDate, HebText);
	}
	
	/**
	 * Returns the Gregorian date from a given Hebrew date,
	 * defined by numbers.
	 * The numbers of Adar A and Adar B are 6 and 7 in a leap year.
	 * In regular year they are both named Adar, use 7.
	 * The year parameter should includes the thousands.
	 */
	public String HebToGreg( int DayH, int MonthH, int YearH) throws ParseException
	{
		int nLengthOfYear ;
	    boolean bLeap ;
	    long dGreg ;
	    int nMonthLen=0 ;
	    boolean bHaser;
	    boolean bShalem ;
	    String fulldate;
	    
	    if ((DayH < 1 )|( DayH > 30)|(MonthH<1)|(MonthH>13)|(YearH <3762))
	    	{return "-1";}
	 	
	    bLeap = IsLeapH(YearH);
	    nLengthOfYear = LengthOfYear(YearH);
		if ((nLengthOfYear == 353) | (nLengthOfYear == 383))
	    {  bHaser = true; }
	    else
	    {  bHaser = false;}
	    if ((nLengthOfYear == 355) | (nLengthOfYear == 385))
	    {  bShalem = true ;}
	    else
	    {  bShalem = false ;}
	    dGreg = Tishrei1(YearH);
	    for( int Month = 1; Month < MonthH ; Month++)
	    {
	    	switch  (Month)
	      {
	    	case 1:
	   		case 5:
	    	case 8:
	    	case 10:
	    	case 12:
	           nMonthLen = 30 ;
	           break;
	        case 4:
	   		case 7:
	    	case 9:
	    	case 11:
	    	case 13:
	            nMonthLen = 29;
	            break;
	         case 6:
	            if (bLeap) 
	            {nMonthLen = 30 ;}   //Adar A (6) will be skipped on non-leap years
	            else
	            { nMonthLen = 0;}
	             break;
		     case 2:                         // Cheshvan, see note above
		       	if (bShalem) { nMonthLen =  30;} else {nMonthLen =  29;}
		       	  break ;
		     case 3:                        // Kislev, see note above
		        if (bHaser) { nMonthLen =  29;} else {nMonthLen = 30;}
		     	break ;
	      }
	    dGreg = DateTime.Add(dGreg, 0, 0, nMonthLen);
	    
	    }
        dGreg = DateTime.Add(dGreg, 0, 0, DayH - 1- deltaH(DayH,MonthH,YearH));
	    fulldate = DateTime.GetDayOfMonth(dGreg) 
	     			+ "/" + DateTime.GetMonth(dGreg) 
	     			+ "/" + DateTime.GetYear(dGreg);
	   	return fulldate;
	}
	
	/**
	 * Returns the length of the Hebrew month for that Hebrew year.
	 * Numbers for the months:
	 * 1 - Tishrei, 2 - Cheshvan, 3 - Kislev, 4 - Tevet, 5- Shevat
	 * 6 - Adar A, 7 - Adar or Adar B, 8 - Nissan, 9 - Iyar
	 * 10 - Sivan, 11- Tamuz, 12 - Av , 13 - Elul.
	 */
	public int HebMonthLength(int Hmonth, int Hyear) throws ParseException
	{
		int nLengthOfYear ;
		switch  (Hmonth)
	      {
	    	case 1:
	   		case 5:
	    	case 8:
	    	case 10:
	    	case 12:
	           return 30 ;

	        case 4:
	   		case 7:
	    	case 9:
	    	case 11:
	    	case 13:
	            return 29;

	        case 6:
	            if (IsLeapH(Hyear))
	            {return 30 ;}   //Adar A (6) will be skipped on non-leap years
	            else
	            { return 0;}
		     case 2:              // Cheshvan
		    	 nLengthOfYear = LengthOfYear(Hyear);
		    	 if ((nLengthOfYear == 355) | (nLengthOfYear == 385))
		 	    {  return 30 ;}
		 	    else
		 	    {  return 29 ;}
		     case 3:              // Kislev
		    	 nLengthOfYear = LengthOfYear(Hyear);
		    	 if ((nLengthOfYear == 353) | (nLengthOfYear == 383))
		 	    {  return 29; }
		 	    else
		 	    {  return 30;}
		     default :
		     	return 0 ;
	      }
	}
	
	/**
	 * Returns true if the Gregorian year is leap year, 
	 * so that February has 29 days.
	 */
	public boolean IsLeapG(int year)
	{
		if (year % 4 == 0)
	        if (year % 100 ==0)
	            if (year % 400 == 0)
	               return true ;
	            else
	               return false ;
	         else
	            return true ;
	    else
	        return false ;
	}
		
	private String GToH(long dGreg, boolean HebText) throws ParseException
	{	
		int nMonthH ;
		int nDateH ;
		boolean bHaser ;
		boolean bShalem ;
		int nMonthLen = 0 ;
	    boolean bWhile  ;
		int	nOneMolad = 29 + (12 / 24) + (793 / (1080 * 24)) ;
		int	nAvrgYear = nOneMolad * (235 / 19) ;
		int nDays = (int)((dGreg - DateTime.DateParse("01/01/1900"))/DateTime.TicksPerDay) ;
	
		nDays +=  2067025  ;  // 2067025 days after creation
		int nYearH = (nDays / nAvrgYear) + 1 ;
		long dTishrei1 = Tishrei1(nYearH);
					
		if (dTishrei1 == dGreg)
		  {
		     nMonthH = 1;
		     nDateH = 1;
		  }
		else
		  {
			if (dTishrei1 < dGreg)
			{
				while (Tishrei1(nYearH + 1) <= dGreg)
				 {nYearH++  ;}
		 	}
			else
			{
			     nYearH-- ;
			     while (Tishrei1(nYearH) > dGreg)
			        {nYearH--;}
			}
		  }
		
		nDays = (int)((dGreg - Tishrei1(nYearH))/ DateTime.TicksPerDay) ;
	    int nLengthOfYear = LengthOfYear(nYearH);
	    if ((nLengthOfYear == 353) | (nLengthOfYear == 383))
	    {  bHaser = true; }
	    else
	    {  bHaser = false;}
	    if ((nLengthOfYear == 355) | (nLengthOfYear == 385))
	    {  bShalem = true ;}
	    else
	    {  bShalem = false ;}
		boolean bLeap = IsLeapH(nYearH);
	    nMonthH = 1 ;
	    
	    do 
	    {
		    switch (nMonthH)
	  		{
		     case 1:
		     case 5:
		     case 8:
		     case 10:
		     case 12:
		    	  nMonthLen =  30;  break;   // 30 day months
		     case 4:
		     case 7:
		     case 9:
		     case 11:
		     case 13:
		    	  nMonthLen =  29;  break;
		     case 6:
		    	  nMonthLen = 30 ; break; //Adar A (6) will be skipped on non-leap years
		     case 2:                         // Cheshvan, see note above
		       	if (bShalem) { nMonthLen =  30;} else {nMonthLen =  29;}
		       	  break ;
		     case 3:                        // Kislev, see note above
		        if (bHaser) { nMonthLen =  29;} else {nMonthLen = 30;}
		     	break ;
		    }    
		    if (nDays >= nMonthLen)
		    {
		      bWhile = true;
		      if (bLeap | (nMonthH != 5))
		        {nMonthH++ ;}
		      else
		          {nMonthH +=  2;}
		      nDays -= nMonthLen;
		    }
		    else
		    { bWhile = false;}
	    }
	     while ( bWhile) ;
		    nDateH = nDays + 1;
		 if ( HebText)
			{return DayName(nDateH) + " " + GetMonthName(nMonthH,nYearH) + " " + YearName(nYearH) ;}
		 else 
			{return nDateH + "/" + nMonthH + "/" + nYearH ;}
	} 
	
	private long Tishrei1(int nYearH) throws ParseException 
	{
		int nMonthsSinceFirstMolad = MonSinceFirstMolad(nYearH);
		int nChalakim = 793 * nMonthsSinceFirstMolad ;
		nChalakim +=  204 ;
		int nHours = nChalakim / 1080 ;
		nChalakim = nChalakim % 1080 ;
		
		nHours +=  nMonthsSinceFirstMolad * 12 ;
		nHours +=  5;
		    
		int nDays = nHours / 24 ;
		nHours = nHours % 24 ;
		
		nDays += 29 * nMonthsSinceFirstMolad ;
		nDays +=  2 ;
		
		int nDayOfWeek = nDays % 7 ;
		
		if ((!IsLeapH(nYearH)) & ( nDayOfWeek == 3) & ((nHours * 1080 + nChalakim) >= (9 * 1080 + 204)))
		  {
		   nDayOfWeek = 5 ;
		   nDays +=  2 ;
		  }
		else
		  {
			if ((IsLeapH(nYearH - 1)) & (nDayOfWeek == 2) & ((nHours * 1080 + nChalakim) >= (15 * 1080 + 589))) 
			  {
				nDayOfWeek = 3 ;
		        nDays++ ;
			  }
		   else
		      {
		   		if (nHours >= 18) 
		   	 	 {
		          nDayOfWeek++;
		          nDayOfWeek %=  7 ;
		          nDays++  ;
		     	 }
		   		if ((nDayOfWeek == 1) | (nDayOfWeek == 4) | (nDayOfWeek == 6)) 
		     	 {
		          nDayOfWeek++  ;
		          nDayOfWeek %=  7;
		          nDays++  ;
		   	 	 }
		      }
		  }
		nDays -=  2067025 ;
		long dTishrei1 = DateTime.DateParse("01/01/1900") ;
		return dTishrei1 + nDays * DateTime.TicksPerDay  ;
	}
	/**
	 * Returns True if the Hebrew year is a Leap year.	
	 * The year parameter should include thousands.
	 */
	public boolean IsLeapH(int hyear )
	{
		hyear = hyear % 19 ;
		if((hyear==0)|(hyear==3)|(hyear==6)|(hyear==8)|(hyear==11)|(hyear==14)|(hyear==17))
			{ return true ;}
		else
		    { return false ;}
	}
	
	private int LengthOfYear(int YearH ) throws ParseException 
	{
		long dThisTishrei1 = Tishrei1(YearH);
		long dNextTishrei1 = Tishrei1(YearH + 1);
   		return (int)((dNextTishrei1 - dThisTishrei1)/ DateTime.TicksPerDay) ;
	}

	private int MonSinceFirstMolad(int nYearH)  
	{
			nYearH  -=  1 ;
			int nMonSinceFirstMolad = (nYearH / 19) * 235 ;
			nYearH %=  19 ;
			nMonSinceFirstMolad +=  12 * nYearH ;
			if (nYearH >= 17) 
			{nMonSinceFirstMolad +=  6 ;}
			else
			{
			  if (nYearH >= 14) 
				{ nMonSinceFirstMolad +=  5;}
			  else
			    {
			  	if (nYearH >= 11) 
			  	{nMonSinceFirstMolad +=  4;}
			    else
			      {
			    	if (nYearH >= 8) 
			    	{nMonSinceFirstMolad +=  3;}
			      else
			        {
			      	if (nYearH >= 6) 
			      	{nMonSinceFirstMolad +=  2;}
			        else
			          {
			        	if (nYearH >= 3) 
			        	{nMonSinceFirstMolad++  ;}
			          }
		            }
			     }
			   }
			}
		return nMonSinceFirstMolad;
	}
	
	private String GetMonthName(int month, int year)
	{
		String [] monthstr = {"", "תשרי", "חשון", "כסלו", "טבת", 
	              "שבט", "אדר", "אדר" , "ניסן", "אייר",
	              "סיון",  "תמוז", "אב", "אלול" } ;
		if(IsLeapH(year))
		{ monthstr[6] = "אדר א" ;
		  monthstr[7] = "אדר ב" ;}
		return monthstr[month] ;
	}
	
	private String DayName(int Day)
	{
		String [] Hday = {"" , "א", "ב" , "ג" , "ד" , "ה", "ו" ,
						  "ז" ,"ח", "ט", "י" , "יא" ,"יב" , "יג",
						  "יד","טו" ,"טז","יז", "יח", "יט", "כ" ,
						  "כא","כב","כג","כד","כה" ,"כו", "כז",
						  "כח","כט" , "ל" };
				
		return Hday[Day] ;
	}
	
	private String YearName(int Year)
	{
		String Hyear = "" ;
		int th ;
		int hun ;
		int ten ;
		int single ;

		th = Year/1000 ;
		hun = (Year - th*1000)/100 ;
		ten = (Year - th*1000 - hun * 100)/10 ;
		single = Year - th*1000 -hun*100 - ten*10 ;
		if (ten < 3) 
			Hyear = ThName( th) + HunName(hun) + DayName(ten*10 + single) ;
		else
			Hyear = ThName( th) + HunName(hun) + TenName(ten)+ DayName(single) ;
			
		return Hyear ;
	}
	
	private String ThName(int Th)
	{
		String [] HY = {"ג'" ,"ד'", "ה'", ""} ;
		return HY[Th - 3] ;		
	}
	
	private String HunName(int Hun)
	{
		String [] HY = {"","ק","ר","ש","ת","תק","תר","תש","תת","תתק"};
		
		return HY[Hun] ;		
	}
	
	private String TenName(int Ten)
	{
		String [] HY =  {"","י","כ","ל","מ","נ","ס","ע","פ","צ" };
		
		return HY[Ten] ;		
	}
	
	private int deltaG(int gday,int gmonth,int gyear)
		{
		if (gyear > 1582) 
		{return 0;}
		else if ((gyear > 1581)&(gmonth > 10))
		{return 0 ;}
		else if ((gyear > 1581)&(gmonth > 9) & (gday > 14))
		{return 0 ;}	
		else if (gyear > 1499)
		{return 10;}
		else if (gyear > 1399) 
		{return 9;}
		else if (gyear > 1299)
		{return 8;}
		else if (gyear > 1099) 
		{return 7;}
		else if (gyear > 999)
		{return 6;}
		else if (gyear > 899)
		{return 5;}
		else if (gyear > 699) 
		{return 4;}
		else if (gyear > 599) 
		{return 3;}
		else if (gyear > 499)
		{return 2;}
		else if (gyear > 399)
		{return 1;}
		else if (gyear > 299)
		{return 0;}
		else if (gyear > 99)
		{return -1;}
		else
		{return 0;}
		
	}
	
	private int deltaH(int gday,int gmonth, int gyear)
		{
		if (gyear > 5343)
		{return 0;}
		else if ((gyear > 5342) & ((gmonth > 1) | (gday > 18)))
		{return 0 ;}
		else if (gyear > 5260)
		{return 10;}
		else if (gyear > 5160)
		{return 9;}
		else if (gyear > 5060)
		{return 8;}
		else if (gyear > 4860) 
		{return 7;}
		else if (gyear > 4760)
		{return 6;}
		else if (gyear > 4660)
		{return 5;}
		else if (gyear > 4460) 
		{return 4;}
		else if (gyear > 4360) 
		{return 3;}
		else if (gyear > 4260)
		{return 2;}
		else if (gyear > 4060)
		{return 1;}
		else if (gyear > 3960)
		{return 0;}
		else if (gyear > 3860)
		{return -1;}
		else
		{return 0;}
		
	}
}
	
/*	private String GetMonthName(int month, int year)
{
    String monthstr = "Tishri";
	if(IsLeapH(year))
	{
		    if(month == 1)
		    {monthstr = "Tishri"; }
		    else if(month == 2)
		    {monthstr = "Heshvan";}
		    else if(month == 3)
		    {monthstr = "Kislev";}
		    else if(month == 4)
		    {monthstr = "Teveth";}
		    else if(month == 5)
		    {monthstr = "Shevat";}
		    else if(month == 6)
		    {monthstr = "Adar A";}
		    else if(month == 7)
		    {monthstr = "Adar B";}
		    else if(month == 8)
		    {monthstr = "Nisan";}
		    else if(month == 9)
		    {monthstr = "Iyar";}
		    else if(month == 10)
		    {monthstr = "Sivan";}
		    else if(month == 11)
		    {monthstr = "Tammuz";}
		    else if(month == 12)
		    {monthstr = "Av";   }
		    else if(month == 13)
		    { monthstr = "Elul";}
	}
else
	{
		    if(month == 1)
		    {monthstr = "Tishri";}
		    else if(month == 2)
		    {monthstr = "Heshvan";}
		    else if(month == 3)
		    {monthstr = "Kislev";}
		    else if(month == 4)
		    {monthstr = "Teveth";}
		    else if(month == 5)
		    {monthstr = "Shevat";}
		    else if(month == 6)  
		    {monthstr = "Adar";}
		    else if(month == 7)
		    {monthstr = "Adar";}
		    else if(month == 8)
		    {monthstr = "Nisan";}
		    else if(month == 9)
		    {monthstr = "Iyar";}
		    else if(month == 10)
		    {monthstr = "Sivan";}
		    else if(month == 11)
		    {monthstr = "Tammuz";}
		    else if(month == 12)
		    {monthstr = "Av";   }
		    else if(month == 13)
		    { monthstr = "Elul";}
	}
return monthstr ;
}*/	
